--in this file is all event handling for the buffTimers


-- List of Blessings with Duration
local blessings = {
	["Blessing of Might"] = 300,
	["Blessing of Kings"] = 300,
	["Blessing of Light"] = 300,
	["Blessing of Wisdom"] = 300,
	["Blessing of Salvation"] = 300,
	["Blessing of Sanctuary"] = 300,	
	["Righteous Fury"] = 1800,  
};

local specials = {
	["Blessing of Freedom"] = 10,
	["Blessing of Protection"] = 10,
	["Blessing of Sacrifice"] = 30,
}

local greaterBlessings = {
	["Greater Blessing of Might"] = 900,
	["Greater Blessing of Kings"] = 900,
	["Greater Blessing of Light"] = 900,
	["Greater Blessing of Wisdom"] = 900,
	["Greater Blessing of Salvation"] = 900,
	["Greater Blessing of Sanctuary"] = 900,
};

local lastSpell;  -- Save Date between Casting Events
local lastTarget;
local roster = AceLibrary("Roster-2.1");

--Core Data for Buff Time
-- structure Key = Unit Name 1 = spell name 2 = start time 3 = duration 4 = faded true/false 5 = for class buffs real target else friend/pet indicator (0 = normale, 1 = friend, 2 = pet)
CPala.buffList = {};

function CPala:EnableBuffTime()
	--Init Vars
	lastSpell = nil;
	lastTarget = nil;	
	
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("UNIT_SPELLCAST_SENT");
	
	self:ScheduleRepeatingEvent(self.CheckBuffTime, 5, self);	-- real update every 5 seconds	
end;


--Save last spell cast click
function CPala:UNIT_SPELLCAST_SENT(p, spell, rank, target)
	if (p ~= "player") then
		return
	end;
	lastSpell = spell;
	lastTarget = target;
end

--sucessfull cast 
function CPala:UNIT_SPELLCAST_SUCCEEDED(p, spell, rank)
	local lastSpell = lastSpell; -- Copy in Case global value change
	local lastTarget = lastTarget;	
	if (p ~= "player") then
		return
	end;
	-- if same then last spell sent we can use the target
	if (spell == lastSpell) then	
	
		--check for special Blessing
		if (specials[spell])  then
			local buffList = self.buffList; -- local copy
			if (self.db.char.buffTime.shorts) then
				-- Schedule Expire Message				
				local shortcut = self.shortcuts[spell]
				local spellTime = specials[spell];
				self:ScheduleEvent(function() CPala:Msg(string.format("%s on %s expire in %i seconds", shortcut, lastTarget, (spellTime / 2))) end, (spellTime / 2));
				self:ScheduleEvent(function() CPala:Msg(string.format("%s on %s has expired", shortcut, lastTarget)) end, spellTime);
			end;	
			if (buffList[lastTarget]) then	-- Overbuffed
				buffList[lastTarget][4] = true;		
			end;
			return;
		end
		
		--check for normal blessing	
		if (blessings[spell]) then
			local buffList = self.buffList; -- local copy
			local needList = self.needList;
			local greaterCount = self.greaterCount;
			local none = self.none;			
			
			local indicator = 0;
			local obj = roster:GetUnitObjectFromName(lastTarget);	
			if (not obj) then
				indicator = 1;
				obj = roster:GetUnitObjectFromName(UnitName("player"));	
			elseif (obj.class == "PET") then
				indicator = 2;
			end;
						
			--special action for double buffs on self
			if (spell == "Righteous Fury") then
				lastTarget = self.selfbuff;		
				indicator = 0;
			end;
			--check if new buff target or overwriting an old buff
			if (buffList[lastTarget] == nil) then
				buffList[lastTarget] = {spell, time(), blessings[spell], false, indicator };  -- create new table
			else
				buffList[lastTarget][1] = spell;  -- reuse table
				buffList[lastTarget][2] = time();
				buffList[lastTarget][3] = blessings[spell];
				buffList[lastTarget][4] = false;				
				buffList[lastTarget][5] = indicator;
			end;
			
			--Remove Need Entry
		--[[	if (needList[lastTarget] and (needList[lastTarget][1] ~= none) ) then
				local needEntry = needList[lastTarget];
				if greaterBlessings[needEntry[1] ] then
					local obj = roster:GetUnitObjectFromName(lastTarget);
					if (obj) then -- Experiment Reduce Class Need Count
						greaterCount[obj.class] = greaterCount[obj.class] - 1;
					end;
				end
				if  (needEntry[1] ~= spell) and (not (strsub(needEntry[1], 9) == spell)) then					
					self:Msg(format("Buffed %s on %s but needed %s!", self.shortcuts[spell], lastTarget, self.shortcuts[needEntry[1] ]));
					needEntry[3] = true; -- wrong buff					
				else
					needEntry[1] = none; -- no need for buff
				end;			
			end;  ]]
			
			if (needList[lastTarget]) and (needList[lastTarget][1] ~= none) then
				local needSpell = needList[lastTarget][1];
			    if (needSpell ~= spell) and (not (strsub(needSpell, 9) == spell)) then
					-- this is here and not in the loop above because we want only one message per class
					self:Msg(format("Buffed %s on %s but needed %s!", self.shortcuts[spell], lastTarget, self.shortcuts[needSpell]));
				end;				
			end;
										
			self:TriggerEvent("CPALA_UPDATE_LIST");
			return;
		end	

		--Check for greater Blessings
		if (greaterBlessings[spell]) then
			local buffList = self.buffList; -- local copy
			local needList = self.needList;
			local none = self.none;		
			local greaterCount = self.greaterCount;
			local indicator = lastTarget;
			
			local obj = roster:GetUnitObjectFromName(lastTarget);	
			--[[if (not obj) then
				return; -- for safety only, activate this if bugs occur
			end; ]]
			local class = obj.class;
			
			if (class == "PET") then 
				class = lastTarget;  -- pets with greater blessing are blessd only solo
				indicator = 2;
			elseif (buffList[lastTarget]) then 	-- Remove Old Direct Spells for this Target
					buffList[lastTarget] = nil;
			end;
			
			--check if new buff target or overwriting an old buff
			if (buffList[class] == nil) then
				buffList[class] = {spell, time(), greaterBlessings[spell], false, indicator };
			else
				buffList[class][1] = spell;
				buffList[class][2] = time();
				buffList[class][3] = greaterBlessings[spell];
				buffList[class][4] = false;
				buffList[class][5] = indicator; -- Set who we realy buffed
			end;
			
			-- check for wrong buff 
			if (needList[lastTarget]) and (needList[lastTarget][1] ~= none) then
				local needSpell = needList[lastTarget][1];
			    if (needSpell ~= spell) and (not (strsub(spell, 9) == needSpell)) then
					-- this is here and not in the loop above because we want only one message per class
					self:Msg(format("Buffed %s on %s but needed %s!", self.shortcuts[spell], lastTarget, self.shortcuts[needSpell]));
				end;				
			end;
						
			--clean up need entrys
		--[[	for target, entry in pairs(needList) do				
				obj = roster:GetUnitObjectFromName(target);
				if (obj) then
					if (obj.class == class) then
						if (entry[1] == spell)  then
							entry[1] = none;  -- correct spell cast		
							greaterCount[obj.class] = greaterCount[obj.class] - 1;							
						else
							entry[3] = true;   -- other then need spell cast							
						end;
					end;
				else
					-- Cleanup none existing entrys
					if  (not target == self.selfbuff) then
						needList[target] = nil;
					end;
				end;
			end; ]]
			
			--Search and delete Overbuffed single buffs
			for target, entry in pairs(buffList) do
				obj = roster:GetUnitObjectFromName(target);
				if (obj) then							
					if (obj.class == class) then						
						entry[4] = true;
					end;
				else  
					-- Cleanup none existing entrys
					if (not self.greaterCount[target]) and (not target == self.selfbuff) then
						buffList[target] = nil;
					end;					
				end;
			end;					
		
			self:TriggerEvent("CPALA_UPDATE_LIST");
			return;
		end				
	
	
	end;
end

local nextUpdate = false;

--Checks if buffs are still active
function CPala:CheckBuffTime()	
	local t = time(); -- current time
	local delta; -- delta time
	local id; -- unit id	
	local countB = 0;
	local countF = 0;
	for target, entry in pairs(self.buffList) do
		countB = countB + 1;
		if (not entry[4]) then
			delta = t - entry[2];	
			if (delta > entry[3]) then
				-- Warn for expired buff
				if self.classList[target] then
					target = self.classList[target]
				end;
				self:Msg(format("%s on %s has expired!", self.shortcuts[entry[1]], target));
				entry[4] = true;
				countF = countF + 1;				
			else				
				--check for faded buffs				
				id = roster:GetUnitIDFromName(target);
				if (id and (delta > 5)) then
					if (not self:HasBuff(id, entry[1])) then
						self:Msg(format("%s on %s has faded!", self.shortcuts[entry[1]], target));
						entry[4] = true;
						countF = countF + 1;
					end;
				elseif ((target == self.selfbuff) and (delta > 5)) then
					if (not self:HasBuffAll("player", entry[1])) then
						self:Msg(format("%s on %s has faded!", self.shortcuts[entry[1]], target));
						entry[4] = true;
						countF = countF + 1;
					end;
				end;
			end;		
		else
			countF = countF + 1;
		end;
	end;	

	if (nextUpdate) then
		self:TriggerEvent("CPALA_UPDATE_LIST"); -- Call Update List with half frequency
		nextUpdate = false;
	else
		nextUpdate = true;
	end;
	self:TriggerEvent("CPALA_UPDATE_SUMMARY", nil, countB - countF, countF, nil); -- Update fu text
end;