-- CPala Core, Basic Stuff Loading, Chat Command Handling, and Shared Events

--[[
Events : 
CPALA_UPDATE_LIST Fired when buffList or needList was updatet
CPALA_UPDATE_SUMMARY Fired when count of needs/buffs/faded update
arg1 = needs
arg2 = buffs
arg3 = faded
arg4 = seal  (string)
]]

--Ace Addon
CPala = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "AceHook-2.1")
CPala.Class = {}; -- for bar and box class

local roster = AceLibrary("Roster-2.1"); -- helper

--Keybinding Text
BINDING_HEADER_CPALA = "CPala Bindings";


--Used Strings for Specials States
CPala.selfbuff = "SelfBuff";
CPala.none = "NONE";

--*****Database*******
CPala:RegisterDB("CPalaDB", "CPalaPC")
CPala:RegisterDefaults("char", {
	-- Need List Options
	defaultBlessings = {
		["WARRIOR"] = "Greater Blessing of Might",
		["PALADIN"] = "Greater Blessing of Might",
		["MAGE"] = "Greater Blessing of Salvation",
		["HUNTER"] = "Greater Blessing of Might",
		["SHAMAN"] = "Greater Blessing of Wisdom",
		["PRIEST"] = "Greater Blessing of Wisdom",
		["ROGUE"] = "Greater Blessing of Might",
		["DRUID"] = "Greater Blessing of Wisdom",
		["WARLOCK"] = "Greater Blessing of Salvation",		
		[CPala.selfbuff] = "Righteous Fury",
	},
	-- Box Options
	buffTime = {
		enabled = true,
		shorts = true,
	},
	buffBox = { 
		x = 100,
		y = -150,	
		--key = 'F',
		enabled = true,
	},	
	sealBox = {
		x = 150,
		y = -150,
		enabled = true,
	},
	actionBox = {
		x = 200,
		y = -200,
		scale = 1,
		type = "ROW",
		spaceingH = 42,		
		spaceingV = 0,
		enabled = true,
	},	
	firstLoad = true,
	boxBorder = true,
	boxLock = false,	
	chatMsg = true,
	tooltip = true,
	-- Menu Options, for later Use maybe
	menu = {
		greater = true,
		normal = true,
		special = true,
		bok = true,
		bosa = true,
		geraterKey = 'G',
		normalKey = 'T',
		specialKey = 'B',
		enabled = true,
	},
	fu = {
		buff = true,
		greater = true,
		need = true,
		ignore = true,
		text = "B:%b F: %f N: %n; %s",
	},
} ) 

CPala.cmdBuffList = { "BoM", "BoK", "BoL", "BoW", "BoS", "BoSA", "GBoM", "GBoK", "GBoL", "GBoW", "GBoS", "GBoSA", CPala.none}

-- short names always better
CPala.shortcuts = {
	["Blessing of Might"] = "BoM",
	["Blessing of Kings"] = "BoK",
	["Blessing of Light"] = "BoL",
	["Blessing of Wisdom"] = "BoW",
	["Blessing of Salvation"] = "BoS",
	["Blessing of Sanctuary"] = "BoSA",
	["Blessing of Freedom"] = "Free",
	["Blessing of Protection"] = "Prot",
	["Blessing of Sacrifice"] = "Sacri",
	["Righteous Fury"] = "Fury",  
	["Greater Blessing of Might"] = "GBoM",
	["Greater Blessing of Kings"] = "GBoK",
	["Greater Blessing of Light"] = "GBoL",
	["Greater Blessing of Wisdom"] = "GBoW",
	["Greater Blessing of Salvation"] = "GBoS",
	["Greater Blessing of Sanctuary"] = "GBoSA",
	["Judgement of the Crusader"] = "JotC",
	["Judgement of Wisdom"] = "JoW",
	["Judgement of Light"] = "JoL",
	["Judgement of Justice"] = "JoJ",
	["Hammer of Justice"] = "HoJ",
	[CPala.none] = CPala.none,
}

-- ***** Init Stuff *******
--Things that only be need to be one time
function CPala:OnInitialize()
	local db = self.db.char; -- Local Copy
	
	self.cbuffcmd.handler = self.cbuffcmd;	 -- we dont want the profile option ...
	self:RegisterChatCommand({"/cbuff", "/cpala"}, self.cbuffcmd);
	self.cbuffcmd.handler = self;
	--create all menu entrys ...
	for name in pairs(self.db.char.defaultBlessings) do
		if (name ~= self.selfbuff) then
			self:Add_Player(name, true);
		end;
	end;
	
	--Init Stuff
	if db.menu.enabled then
		self:InitMenus(); -- Load Up the Menus
	end;
	if db.sealBox.enabled then
		self:InitSealBox();
	end;
	--[[if db.buffTime.enabled then
		self:InitBuffTime(); -- no init needed
	end;]]
	if  db.buffBox.enabled then
		self:InitBuffNeed();
	end;
	if db.actionBox.enabled then
		self:InitActionBar();
	end;
	
	-- Remove not acitve buffs
	if (not db.menu.bok) then	
		for k,v in pairs(self.cmdBuffList) do
			if(v == "BoK" or v == "GBoK") then
				table.remove(self.cmdBuffList, k);		
			end
		end
		self.cbuffcmd.args.templates.args.gbok = nil;
	end;
		
	if (not db.menu.bosa) then
		for k,v in pairs(self.cmdBuffList) do
			if(v == "BoSA" or v == "GBoSA") then
				table.remove(self.cmdBuffList, k);		
			end
		end
		self.cbuffcmd.args.templates.args.gbosa = nil;
	end;
end


local aura_test1, aura_test2 = false; -- save time in UNIT_AURA event ...

function CPala:OnEnable()
	local db = self.db.char; -- Local Copy
	
	-- Start Stuff
	--[[if db.menu.enabled then
		self:EnableMenus(); 
	end]]
	if db.buffTime.enabled then	
		self:EnableBuffTime();
	end;
	if db.sealBox.enabled then
		self:EnableSealBox();
	end;
	if db.buffBox.enabled then
		self:EnableBuffNeed();
	end;
	--[[if db.actionBox.enabled then
		self:EnableActionBar();
	end;]]	
	if db.tooltip then
		self:EnableTooltip();
	end;	
	aura_test1 = self.db.char.sealBox.enabled;
	aura_test2 = self.db.char.buffBox.enabled;
	self:RegisterEvent("UNIT_AURA");
	self:RegisterEvent("UPDATE_BINDINGS");
	
	if db.firstLoad then
		self:Print("First Loaded, Please Set Keybindings!");
		db.firstLoad = false;
	end;
end

function CPala:EnableTooltip()
	--Tootip Display is Used by needList AND buffList
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	self:SecureHook(GameTooltip, "SetUnit", "UPDATE_MOUSEOVER_UNIT");	
end;

function CPala:DisableTooltip()
	--Tootip Display is Used by needList AND buffList
	self:UnregisterEvent("UPDATE_MOUSEOVER_UNIT");
	self:Unhook(GameTooltip, "SetUnit");	
end;

function CPala:OnDisable()
	self:Print("You Should not disable this (not implemented)");	
end


function CPala:UNIT_AURA(unit)
	if (aura_test1) then
		self:SealBoxAura(unit);
	end
	if (aura_test2) then
		self:BuffNeedAura(unit);
	end;
end;


function CPala:UPDATE_BINDINGS()
	local db = self.db.char; -- Local Copy
	if InCombatLockdown() then
		--canot update during combat!
		self:ScheduleLeaveCombatAction(self.UPDATE_BINDINGS, self);
		return;
	end;
	--[[if db.menu.enabled then
		self:SetMenuKeys() 
	end]]
	--[[if db.buffTime.enabled then	
		self:SetBuffTimeKeys();
	end;]]
	--[[if db.sealBox.enabled then
		self:SetSealBoxKeys();
	end;]]
	if db.buffBox.enabled then
		self:SetBuffBoxKeys();
	end;
	if db.actionBox.enabled then
		self:SetActionBarKeys();
	end;	
end;

--**** Utility Functions ****
-- Check if the unit has a specific buff, only checks self-castable buffs ;)
function CPala:HasBuff(unit, buff)
	local i = 1;	
	local buff_name = UnitBuff(unit, i, true);
	while (buff_name) do
		if buff_name == buff then 
			return true;
		end;
		i = i + 1;
		buff_name = UnitBuff(unit, i, true);
	end;
	return false;
end;

-- Check if the unit has a specific buff,
function CPala:HasBuffAll(unit, buff)
	local i = 1;		
	local buff_name = UnitBuff(unit, i, false);
	while (buff_name) do		
		if buff_name == buff then 			
			return true;
		end;
		i = i + 1;
		buff_name = UnitBuff(unit, i, false);
	end;
	return false;
end;

--Nice Time String
function CPala:FormatSeconds(sec)
	local minute = floor(sec / 60);
	sec = mod(sec, 60);	
	return format("%i:%.2i", minute, sec);
end;

--Print Message with check if disabled or not 
function CPala:Msg(text)
	if self.db.char.chatMsg then
		self:Print(text);
	end;
end;

--******* Shared Events ******
function CPala:UPDATE_MOUSEOVER_UNIT()
	if UnitExists("mouseover") and UnitCanAssist("mouseover", "player")  then
		local entry = self.buffList[UnitName("mouseover")];
		local tip = GameTooltip;		
		if (not entry) then
			local _, class = UnitClass("mouseover");
			entry = self.buffList[class];
		end;
		if entry then
			local spell = entry[1];
			if (not entry[4]) then
				if (strfind(spell, "Greater")) and UnitPlayerOrPetInRaid("mouseover") and  (not self:HasBuff("mouseover", spell)) then
					tip:AddLine(format("%s faded, Group buff lasting %s", self.shortcuts[spell], self:FormatSeconds(entry[3] - (time() - entry[2]))));
				else
					tip:AddLine(format("%s lasting %s", self.shortcuts[spell], self:FormatSeconds(entry[3] - (time() - entry[2]))));
				end;
			else
				tip:AddLine(format("%s faded", self.shortcuts[spell]));
			end;		
		end;
		entry = self.needList[UnitName("mouseover")];
		if (entry) then
			if (entry[1] ~= self.none) then
				tip:AddLine(format("Need %s", self.shortcuts[entry[1]]));
			end;
		end;
		tip:Show();
	end;
end;


--*******Chat Command *********
CPala.cbuffcmd = {
    type = 'group',
    name = 'cbuff',
    args = {
		--templates first
		templates =  {
			type = 'group',
			name = 'Templates',
			desc = 'Setup all Class Buffs in On Click',
			order = 5,
			args = {
				gbok = {
					type = 'execute',
					name = 'GBoK',
					desc = 'GBoK for all',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
					   CPala:SetClassBuffs("GBoK", "GBoK", "GBoK", "GBoK", "GBoK", "GBoK", "GBoK", "GBoK", "GBoK");
					   CPala:Print("Setting all to GBoK");
						CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
				gbol = {
					type = 'execute',
					name = 'GBoL',
					desc = 'GBoL for all',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
					   CPala:SetClassBuffs("GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL");
					   CPala:Print("Setting all to GBoL");
					   CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
				gbosa = {
					type = 'execute',
					name = 'GBoSA',
					desc = 'GBoSA for all',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
					   CPala:SetClassBuffs("GBoSA", "GBoSA", "GBoSA", "GBoSA", "GBoSA", "GBoSA", "GBoSA", "GBoSA", "GBoSA");
					   CPala:Print("Setting all to GBoSA");
					   CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
				gbos = {
					type = 'execute',
					name = 'GBoS',
					desc = 'GBoS for all, GBoL for Warrior',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
					   CPala:SetClassBuffs("GBoL", "GBoS", "GBoS", "GBoS", "GBoS", "GBoS", "GBoS", "GBoS", "GBoS");
					   CPala:Print("Setting all to GBoS and GBoL for Warrior");
					   CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
				gbow = {
					type = 'execute',
					name = 'GBoW/GBoM',
					desc = 'GBoW and GBoM',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
					   CPala:SetClassBuffs("GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL", "GBoL");
					   CPala:Print("Setting GBoW and GBoM");
					   CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
				solo = {
					type = 'execute',
					name = 'Solo',
					desc = 'Solo Buffs',
					func = function() 
						--CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
						if (CPala.db.char.menu.bok) then
							CPala:SetClassBuffs("BoM", "BoM", "BoK", "BoK", "BoK", "BoK", "BoK", "BoM", "BoM");
						else
							CPala:SetClassBuffs("BoM", "BoM", "BoW", "BoW", "BoW", "BoW", "BoW", "BoM", "BoM");
						end;
						CPala:Print("Setting Solo buffs");
						CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					
					end,
				},
				gsolo = {
					type = 'execute',
					name = 'GreaterSolo',
					desc = 'Greater Solo Buffs',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
						if (CPala.db.char.menu.bok) then
							CPala:SetClassBuffs("GBoM", "GBoM", "GBoK", "GBoK", "GBoK", "GBoK", "GBoK", "GBoM", "GBoM");
						else
							CPala:SetClassBuffs("GBoM", "GBoM", "GBoW", "GBoW", "GBoW", "GBoW", "GBoW", "GBoM", "GBoM");
						end;
					   CPala:Print("Setting Grater Solo buffs");
					   CPala.db.char.defaultBlessings[CPala.selfbuff] = "Righteous Fury";
					   CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
				groupTank = {
					type = 'execute',
					name = 'GroupTank',
					desc = 'Group Tanking buffs',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
						if (CPala.db.char.menu.bok) then
							CPala:SetClassBuffs("GBoM", "GBoK", "GBoW", "GBoW", "GBoW", "GBoS", "GBoS", "GBoM", "GBoM");
						else
							CPala:SetClassBuffs("GBoM", "GBoW", "GBoW", "GBoW", "GBoW", "GBoS", "GBoS", "GBoM", "GBoM");
						end
					   CPala:Print("Setting Grop Tank buffs");
					   CPala.db.char.defaultBlessings[CPala.selfbuff] = "Righteous Fury";
					   CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
				groupHeal = {
					type = 'execute',
					name = 'GroupHeal',
					desc = 'Group Heal buffs',
					func = function() 
					   --CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
						if (CPala.db.char.menu.bok) then
							CPala:SetClassBuffs("GBoK", "GBoW", "GBoW", "GBoW", "GBoW", "GBoS", "GBoS", "GBoM", "GBoM");
						else
							CPala:SetClassBuffs("GBoL", "GBoW", "GBoW", "GBoW", "GBoW", "GBoS", "GBoS", "GBoM", "GBoM");
						end
					   CPala:Print("Setting Grop Heal buffs");
					   CPala.db.char.defaultBlessings[CPala.selfbuff] = CPala.none;
					   CPala:CheckBuffTime();
						CPala:CheckBuffNeed(); 
					end,
				},
			},
		},
		--then all class stuff
		header0 = {
			type = 'header',
			order = 15,
		},
		--PlayerList		
		selfbuff = {
			type = 'text',
			name = CPala.selfbuff,
			desc = "Setup Default Buff for Self",
			get = function() return CPala.shortcuts[CPala.db.char.defaultBlessings[CPala.selfbuff]] end,
			set = function(msg) CPala.db.char.defaultBlessings[CPala.selfbuff] = CPala:Get_By_Shortcut(msg); end,
			validate = { "Fury", CPala.none },
			order = 19,
		},		
		header1 = {
			type = 'header',
			order = 25,
		},
		--Options
		add = {
			type = 'text',
			name = 'Add',
			desc = 'Add a Specific Player',
			get = false,
			set = 'Add_Player',
			get = function() 
				--display current Target Name as default if it does make sense
				if UnitExists("target") and UnitPlayerOrPetInRaid("target") and (not CPala.db.char..defaultBlessings[UnitName("target")]) then
					return UnitName("player");
				end;
			end,
			usage = '<name>',
			order = 30,
		},
		remove = {
			type = 'text',
			name = 'Remove',
			desc = 'Remove a Specific Player',
			get = false,
			set = 'Remove_Player',
			usage = '<target>',
			validate = {},
			order = 30,
		},
		reset = {
			type = 'execute',
			name = 'Reset Data',
			desc = 'Reset Saved Buff Data',
			func = 'Reset',
			order = 31,
		},
		update = {
			type = 'execute',
			name = 'Update Needs',
			desc = 'Force a Need List Update',
			func = 'CheckBuffNeed',
			order = 31,
		},		
		header2 = {
			type = 'header',
			order = 35,
		},				
		options = {
			type = 'group',
			name = 'Options',
			desc = 'Set General Options',
			disabled = InCombatLockdown,
			order = 40,
			args = {
				lock = {
					order = 10,
					type = 'toggle',
					name = 'Lock',
					desc = 'Lock/Unlock Box Draging',
					get = function() return CPala.db.char.boxLock end,
					set = 'Set_Lock',
					disabled = InCombatLockdown,
				},
				border = {
					order = 11,
					type = 'toggle',
					name = 'Border',
					desc = 'Toggle Box-Border Visiblity',
					get = function() return CPala.db.char.boxBorder end,
					set = 'Set_Border',
					disabled = InCombatLockdown,
				},
				reset = {
					order = 12,
					type = 'execute',
					name = 'Reset Positions',
					desc = 'Reset Box Positions',
					func = function() 
						CPala.needbox:SetPos(100, -100);
						CPala.db.char.buffBox.x = 100;
						CPala.db.char.buffBox.y = -100;
						CPala.sealbox:SetPos(150, -150);
						CPala.db.char.sealBox.x = 150;
						CPala.db.char.sealBox.y = -150;
						CPala.actionBox:SetPos(200, -200);
						CPala.db.char.actionBox.x = 200;
						CPala.db.char.actionBox.y = -200;
					end,
				},
				header1 = {
					type = 'header',
					order = 20,
				},	
				bok = {
					order = 30,
					type = 'toggle',
					name = 'BoK',
					desc = 'Enable/Disable BoK in Blessing Menu',
					get = function() return CPala.db.char.menu.bok end,
					set = function(value) 
						CPala.db.char.menu.bok = value; 
						if (value) then
							CPala:Print("BoK now active - need UI Reload");
						else
							CPala:Print("BoK now inactive - need UI Reload");
						end;
					end,
				},
				bosa = {
					order = 31,
					type = 'toggle',
					name = 'BoSA',
					desc = 'Enable/Disable BoSA in Blessing Menu',
					get = function() return CPala.db.char.menu.bosa end,
					set = function(value) 
						CPala.db.char.menu.bosa = value; 
						if (value) then
							CPala:Print("BoSA now active - need UI Reload");
						else
							CPala:Print("BoSA now inactive - need UI Reload");
						end;
					end,
				},
				chatMsg	= {						
					type = 'toggle',
					name = 'Chat Messages',
					order = 32,
					desc = 'Enable/Disable Chat Messages',
					get = function() return CPala.db.char.chatMsg end,
					set = function(value) 
						CPala.db.char.chatMsg = value; 						
					end,
				},
				tooltip	= {						
					type = 'toggle',
					name = 'Tooltip',
					order = 33,
					desc = 'Enable/Disable Tooltip Display',
					get = function() return CPala.db.char.tooltip end,
					set = function(value) 
						CPala.db.char.tooltip = value; 		
						if (value) then
							CPala:EnableTooltip();
						else
							CPala:DisableTooltip();
						end;
					end,
				},
				shorts = {						
					type = 'toggle',
					name = 'Short Blessings',
					order = 34,					
					desc = 'Enable/Disable Short Blessing Duration Chat Message (Freedom, Protection, Sacrifice)',
					get = function() return CPala.db.char.buffTime.shorts end,
					set = function(value) 
						CPala.db.char.buffTime.shorts = value; 						
					end,
				},
				header2 = {
					type = 'header',
					order = 40,
				},
				bar = {
					type = 'group',
					name = 'Bar',
					desc = 'Action Bar Options',
					disabled = InCombatLockdown,
					order = 41,
					args = {
						scale = {
							type = 'range',
							name = 'Bar Scale',
							desc = 'Sets the Action Button Scale',
							min = 0.1,
							max = 3.0,
							step = 0.1,
							get = function()
								return CPala.db.char.actionBox.scale;
							end,
							set = 'SetButtonScale',
							disabled = InCombatLockdown,
						},
						spaceingH = {
							type = 'range',
							name = 'Spaceing Horziontal/Radius',
							desc = 'Type Row : Horizontal Spaceing, Circle: Radius',
							min = -200,
							max = 200,
							step = 1,
							get = function()
								return CPala.db.char.actionBox.spaceingH;
							end,
							set = 'SetButtonSpaceingH',
							disabled = InCombatLockdown,
						},
						spaceingV = {
							type = 'range',
							name = 'Spaceing Vertical/Angle',
							desc = 'Type Row : Vertical Spaceing, Circle: Used Angle Range',
							min = -360,
							max = 360,
							step = 1,
							get = function()
								return CPala.db.char.actionBox.spaceingV;
							end,
							set = 'SetButtonSpaceingV',
							disabled = InCombatLockdown,
						},
						type = {
							type = 'text',
							name = 'Bar Layout',
							desc = 'Postion Style of Bar',
							get = function() return CPala.db.char.actionBox.type end,
							set = 'SetButtonPositionType',
							validate = { "ROW", "ROUND" },	
							disabled = InCombatLockdown,
						},
					},
				},
				keys = {
					type = 'group',
					name = 'Keybindings',
					desc = 'Key Configuration',
					disabled = InCombatLockdown,
					order = 42,
					args = {
						blessing = {
							type = 'text',
							name = 'Blessing Menu',
							desc = 'Key for Normal Blessing Menu',
							get = function() return CPala.db.char.menu.normalKey end,
							set = 'SetSoloKey',
							usage = '<key>',
							disabled = InCombatLockdown,
						},
						greater = {
							type = 'text',
							name = 'Greater Menu',
							desc = 'Key for Greater Blessing Menu',
							get = function() return CPala.db.char.menu.geraterKey end,
							set = 'SetGreaterKey',							
							usage = '<key>',
							disabled = InCombatLockdown,
						},
						specials = {
							type = 'text',
							name = 'Specials Menu',
							desc = 'Key for Special Blessing Menu',
							get = function() return CPala.db.char.menu.specialKey end,
							set = 'SetSpecialKey',							
							usage = '<key>',
							disabled = InCombatLockdown,
						},
					--[[	buff = {
							type = 'text',
							name = 'Auto Buff',
							desc = 'Key for Auto Buff',
							get = function() return CPala.db.char.buffBox.key end,
							set = 'SetBuffKey',		
							usage = '<key>',
							disabled = InCombatLockdown,
						},	]]
						bar = {
							type = 'text',
							name = 'ActionBar plz use Blizz Default',
							desc = 'ActionBar Use Default Action Button Bindings',
							get = function() return "nil" end,
							set = 'SetSpecialKey',		
							usage = '<key>',
							disabled = true,
						},
					},
				},
				modules = {
					type = 'group',
					name = 'Module Config',
					desc = 'Enabling/Disabling Modules',					
					order = 43,
					args = {
						sealBox = {						
							type = 'toggle',
							name = 'SealBox',
							order = 5,
							desc = 'Enable/Disable SealBox (Box when No Seal, Judgement List on Target and Cleanse Reminder)',
							get = function() return CPala.db.char.sealBox.enabled end,
							set = function(value) 
								CPala.db.char.sealBox.enabled = value; 
								if (value) then
									CPala:InitSealBox();
									CPala:EnableSealBox();
								else
									CPala:Print("SealBox now inactive - need UI Reload to take effect");
								end;
							end,
						},
						buffBox = {						
							type = 'toggle',
							name = 'BuffBox',
							order = 5,
							desc = 'Enable/Disable BuffBox (Buff Need List)',
							get = function() return CPala.db.char.buffBox.enabled end,
							set = function(value) 
								CPala.db.char.buffBox.enabled = value; 
								if (value) then
									CPala:InitBuffNeed();
									CPala:EnableBuffNeed();									
								else
									CPala:Print("BuffBox now inactive - need UI Reload to take effect");
								end;
							end,
						},
						actionBar = {						
							type = 'toggle',
							name = 'ActionBar',
							order = 5,
							desc = 'Enable/Disable ActionBar',
							get = function() return CPala.db.char.actionBox.enabled end,
							set = function(value) 
								CPala.db.char.actionBox.enabled = value; 
								if (value) then
									CPala:InitActionBar();
									--CPala:EnableActionBar();
								else
									CPala:Print("ActionBar now inactive - need UI Reload to take effect");
								end;
							end,
						},
						menus= {						
							type = 'toggle',
							name = 'Menus',
							order = 5,
							desc = 'Enable/Disable Blessing Menus',
							get = function() return CPala.db.char.menu.enabled end,
							set = function(value) 
								CPala.db.char.menu.enabled = value; 
								if (value) then
									CPala:InitMenus();
									--CPala:EnableMenus();
								else
									CPala:Print("Menus now inactive - need UI Reload to take effect");									
								end;
							end,
						},
						buffTime= {						
							type = 'toggle',
							name = 'Buff Timer',
							order = 5,
							desc = 'Enable/Disable BuffTimer',
							get = function() return CPala.db.char.buffTime.enabled end,
							set = function(value) 
								CPala.db.char.buffTime.enabled = value; 
								if (value) then
									--CPala:InitBuffTime();
									CPala:EnableBuffTime();
								else
									CPala:Print("BuffTimer now inactive - need UI Reload to take effect");									
								end;
							end,
						},		
						
					},
				},
			},			
		},
		header3 = {
			type = 'header',
			order = 60,
		},
		
		
    },
}; 

function CPala:Get_By_Shortcut(buff)
	for key, short in pairs(self.shortcuts) do
		if string.lower(short) == string.lower(buff) then
			return key;
		end;
	end;
	return buff;
end;

--Helper for Esay Template Writing ;)
function CPala:SetClassBuffs(war, pala, prie, drui, sham, mage, warl, hun, rog)
	local defaultBlessings = self.db.char.defaultBlessings;
	defaultBlessings["WARRIOR"] = self:Get_By_Shortcut(war);
	defaultBlessings["PALADIN"] = self:Get_By_Shortcut(pala);
	defaultBlessings["MAGE"] = self:Get_By_Shortcut(mage);
	defaultBlessings["HUNTER"] = self:Get_By_Shortcut(hun);
	defaultBlessings["SHAMAN"] = self:Get_By_Shortcut(sham);
	defaultBlessings["PRIEST"] = self:Get_By_Shortcut(prie);
	defaultBlessings["ROGUE"] = self:Get_By_Shortcut(rog);
	defaultBlessings["DRUID"] = self:Get_By_Shortcut(drui);
	defaultBlessings["WARLOCK"] = self:Get_By_Shortcut(warl);
end;

-- Adds a Special entry to the defaultBlessing list, option silent is used for default entrys during startup
function CPala:Add_Player(name, silent)
	if name then
		local defaultBlessings = self.db.char.defaultBlessings;
		if self.cbuffcmd.args[name] then
			self:Print("Cant Add Player/Menu Entry " .. name .. " because it already exists!");
			return;
		end;
		local obj;
		if (not silent) then		
			obj = roster:GetUnitObjectFromName(name);
			if (not obj) then
				obj = roster:GetUnitObjectFromName(string.lower(name));
			end;
			if (not obj) then
				obj = roster:GetUnitObjectFromName(format("%s%s", string.upper(string.sub(name, 1, 1)), string.sub(name, 2)));
			end
		end;
		
		if ((not obj) and (not silent)) then
			self:Print(format("Cant find %s in your group/raid, maybe wrong spelling/case?", name));			
		end;
	
		-- Create Blessing List entry
		if (not silent) and (obj) then
			name = obj.name --get real name off list
		end;
		if (not silent) then
			defaultBlessings[name] = self.none;			
		end;
			
		-- create new menu entry
		self.cbuffcmd.args[self.classList[name]] = {};
		local menu = self.cbuffcmd.args[self.classList[name]];
		menu["type"] = 'text';
		if self.greaterCount[name] then
			-- Make the Class Name Look Better
			menu["name"] = self.classList[name]; --strsub(name, 1,1) .. string.lower(strsub(name,2));
		else
			menu["name"] = name;
		end;
		menu["desc"] = "Setup Default Buff for " .. name;
		menu["get"] = function() return CPala.shortcuts[defaultBlessings[name]] end;
		menu["set"] = function(msg) 
			defaultBlessings[name] = CPala:Get_By_Shortcut(msg); 
			CPala:CheckBuffTime();
			CPala:CheckBuffNeed(); 
		end;
		menu["validate"] = self.cmdBuffList;
			
		if (not self.greaterCount[name]) then
			menu["order"] = 20;
			tinsert(self.cbuffcmd.args.remove.validate, name);
		else
			menu["order"] = 10;
		end;
			
		if (not silent) then
			self:Print(format("Added Player %s  buff set to None", name));
		end
	else		
		self:Print("Usage : /cbuff add <target>");		
	end;
end;

function CPala:Remove_Player(name)	
	local defaultBlessings = self.db.char.defaultBlessings;
	if defaultBlessings[name] then
		-- Remove Data and Menu Entry
		local args = self.cbuffcmd.args;
		defaultBlessings[name] = nil;
		args[name] = nil;
		-- Remove Remove Entry ;)
		for k,v in pairs(args.remove.validate) do
			if(v == name) then
				table.remove(args.remove.validate, k);		
			end
		end
		self:Print("Removed entry for " .. name);
	else
		self:Print("Can Not Remove because not found;)");
	end;
end;

function CPala:Reset()
	self.buffList = {};
	self.needList = {};
	self:CheckBuffNeed();
end;

function CPala:Set_Lock(value)
	self.db.char.boxLock = value;
	if (value) then
		self.needbox:Lock();
		self.sealbox:Lock();
		self.actionBox:Hide();
	else
		self.needbox:Unlock();
		self.sealbox:Unlock();
		self.actionBox:Show();
	end;
end;

function CPala:Set_Border(value)
	self.db.char.boxBorder = value;
	self.needbox:SetBorderVisible(value);	
	self.sealbox:SetBorderVisible(value);	
end;